class InvalidJwtSignatureException(Exception):
    pass


class ExpireTokenException(Exception):
    pass


class InvalidToken(Exception):
    pass


class PublicKeyException(Exception):
    pass


class InvalidPublicKey(PublicKeyException):
    pass


class FailToLoadPublicKeyException(PublicKeyException):
    pass


class NotExistedKey(PublicKeyException):
    pass


class PublicRequestException(PublicKeyException):
    pass
